require 'aws-sdk-core'
require 'fleetctl/setup'

module Fleetctl::Setup
  module AWS
    class CloudFormationSearch
      class Result
        def initialize(resource)
          @resource = resource
        end

        def description
          "CloudFormation Stack: #{@resource.stack_name}"
        end

        def instances
          [].tap do |instances|
            ec2.describe_instances(filters: [filter]).each do |response|
              response.reservations.each do |res|
                instances.concat res.instances
              end
            end
          end
        end

        private

        def ec2
          @ec2 ||= Aws::EC2::Client.new
        end

        def filter
          { name: 'tag:aws:cloudformation:stack-id', values: [@resource.stack_id] }
        end
      end

      class <<self
        def search(search_term)
          new(search_term).results
        end
      end

      def initialize(search_term)
        @search_term = search_term
      end

      def results
        filtered_resources.map { |r| Result.new(r) }
      end

      private

      def filtered_resources
        all_resources.select do |stack|
          /#{@search_term}/i.match(stack.stack_name)
        end
      end

      def all_resources
        @all_resources ||= Aws::CloudFormation::Client.new.describe_stacks.stacks
      end
    end
  end
end
