require 'optparse'
require 'fleetctl/setup'
require 'fleetctl/setup/aws/search'

module Fleetctl::Setup
  class CLI
    def initialize(argv=ARGV)
      @options = {}
      @argv = argv
    end
    attr_reader :options

    def parse_opts!
      opt_parser = OptionParser.new do |opts|
        opts.banner = "Usage: fleetctl-setup [search-term]"

        opts.on_tail '-h', '--help', 'show this message' do
          warn opts
          exit
        end
      end
      opt_parser.parse!(@argv)

      if @argv.empty?
        warn "ERROR: You must supply a keyword or ID to pick a cluster.\n\n"
        puts opt_parser
        exit
      end
      options[:search_term] = @argv.join(' ')
      options.freeze
    end

    def instance_ips
      return @instance_ips if @instance_ips

      results = AWS::Search.search(options[:search_term])
      if results.empty?
        warn "No stack results found for '#{options[:search_term]}'"
        exit 1
      end

      choice = user_choice(results)
      warn "Looking up instances for #{choice.description}"

      @instance_ips = choice.instances.map(&:public_ip_address).compact
      return @instance_ips
    end

    private

    def user_choice(choices)
      return nil if choices.empty?
      return choices.first if choices.length == 1
      choices.each_with_index do |choice, i|
        $stderr.puts "#{i+1}. #{choice.description}"
      end
      choice_index = prompt_int('Please enter a number to select an option: ', 1..choices.length)
      choices[choice_index - 1]
    end

    def prompt(message, format=nil)
      begin
        $stderr.print message
        response = $stdin.gets.chomp
      end while format && !format.match(response)
      response
    end

    def prompt_int(message, range=nil)
      begin
        int = Integer(prompt(message, /\A\d+\z/))
      end while range && !range.include?(int)
      int
    end
  end
end
